// ----------------------------------------------------------------------------
// Since I've started coding a few indicators/experts
// I was asked about donations:
//
// Yes, you may donate to me via
// paypal@webtemp.org
//
// (although I don't really care)
// It's just for motivation. ;-) Do it if you find the stuff useful.
// ----------------------------------------------------------------------------

#property copyright "mail@webtemp.org"
#property link      ""

#property indicator_chart_window

extern string TimeFrame = "Current time frame";
extern int    MaxBars   = 500;        // Bar count to be processed (to save CPU time)
extern double MinFactor = 1.5;           // Minimum multiple of ATR a breakout candle must have to cause a line

int MaxLines = 2000;
double L[2000], S[2000];
datetime LT[2000], ST[2000];
int Trend;

string indicatorFileName;
int    timeFrame;

int init() {
  for(int i = 0; i < MaxLines; i++) {
    L[i] = 0;
    S[i] = 0;
  }
  Trend = 0;
        indicatorFileName = WindowExpertName();
        timeFrame         = stringToTimeFrame(TimeFrame);
  return(0);
}

int deinit() {
  for(int j = 0; j < MaxLines; j++) {
    string n = "ZL"+j;
    ObjectDelete(n);
    n = "ZS"+j;
    ObjectDelete(n);
  }
  for(j = 0; j < Trend; j++) {
    n = "ZT"+j;
    ObjectDelete(n);
  }
  init();
  return(0);
}

int start() {
   if (timeFrame!=Period()) { iCustom(NULL,timeFrame,indicatorFileName,"",MaxBars,MinFactor,0,0); return(0); }
  static datetime latest = 0;
  int changed = Bars - IndicatorCounted();
  if (changed < 2) return(0);
  
  for(int i = MathMin(changed-1, MaxBars); i > 0; i--) {
    if (latest >= Time[i]) continue;
    latest = Time[i];

    for(int j = 0; j < MaxLines; j++) {
      if ((L[j] > 0) && (Open[i-1] <= L[j])) {
        string n = "ZT"+Trend;
        ObjectDelete(n);
        ObjectCreate(n, OBJ_TREND, 0, LT[j], L[j], Time[i-1]-1, L[j]);
        ObjectSet(n, OBJPROP_COLOR, Silver);
        ObjectSet(n, OBJPROP_RAY, false);
        ObjectSet(n, OBJPROP_BACK, true);
        L[j] = 0;
        Trend++;
      }
      if ((S[j] > 0) && (Open[i-1] >= S[j])) {
        n = "ZT"+Trend;
        ObjectDelete(n);
        ObjectCreate(n, OBJ_TREND, 0, ST[j], S[j], Time[i-1]-1, S[j]);
        ObjectSet(n, OBJPROP_COLOR, Silver);
        ObjectSet(n, OBJPROP_RAY, false);
        ObjectSet(n, OBJPROP_BACK, true);
        S[j] = 0;
        Trend++;
      }
    }

    double r = MathAbs(Close[i+1] - Close[i+2]) * MinFactor;
    double h = (Close[i+1] + Close[i]) / 2;
    if (Close[i] - Close[i+1] > r) {
      for(j = 0; j < MaxLines; j++) {
        if (L[j] == h) break;
        if (L[j] != 0) continue;
        L[j] = Open[i];
        LT[j] = Time[i];
        break;
      }
    }

    if (Close[i] - Close[i+1] < -r) {
      for(j = 0; j < MaxLines; j++) {
        if (S[j] == h) break;
        if (S[j] != 0) continue;
        S[j] = Open[i];
        ST[j] = Time[i];
        break;
      }
    }
  }
  
  for(j = 0; j < MaxLines; j++) {
    n = "ZL"+j;
    if (L[j] == 0) {
      ObjectDelete(n);
      continue;
    } 
    ObjectCreate(n, OBJ_FIBO, 0, 0, 0);
    ObjectSet(n, OBJPROP_FIBOLEVELS, 1);
    ObjectSet(n, OBJPROP_LEVELCOLOR, Green);
    ObjectSet(n, OBJPROP_TIME1, LT[j]);
    ObjectSet(n, OBJPROP_PRICE1, L[j]);
    ObjectSet(n, OBJPROP_TIME2, Time[0]);
    ObjectSet(n, OBJPROP_PRICE2, L[j]);
    ObjectSet(n, OBJPROP_BACK, true);
  }

  for(j = 0; j < MaxLines; j++) {
    n = "ZS"+j;
    if (S[j] == 0) {
      ObjectDelete(n);
      continue;
    } 
    ObjectCreate(n, OBJ_FIBO, 0, 0, 0);
    ObjectSet(n, OBJPROP_FIBOLEVELS, 1);
    ObjectSet(n, OBJPROP_LEVELCOLOR, Red);
    ObjectSet(n, OBJPROP_TIME1, ST[j]);
    ObjectSet(n, OBJPROP_PRICE1, S[j]);
    ObjectSet(n, OBJPROP_TIME2, Time[0]);
    ObjectSet(n, OBJPROP_PRICE2, S[j]);
    ObjectSet(n, OBJPROP_BACK, true);
  }

  return(0);
}

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
//
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
return(s);
}